#include "ancova.hpp"

double ancova(gsl_vector *obs, gsl_matrix *par_h0, gsl_matrix *par_h1)
{
  if (obs->size != par_h0->size1 || obs->size != par_h1->size1)
  {
    cout << "Error: non consistent observation number in linear regression!"<<endl;
    return 2.0;
  }
  if (obs->size <= par_h1->size2)
  {
    cout << "Warning: too few observations in linear regression."<<endl;
    return 2.0;
  }
  double res0, res1;
  size_t fd0, fd1;
  gsl_multifit_linear_workspace *ws;
  gsl_vector *c;
  gsl_matrix *cov;
  ws=gsl_multifit_linear_alloc (obs->size, par_h0->size2);
  c=gsl_vector_alloc(par_h0->size2);
  cov=gsl_matrix_alloc(par_h0->size2, par_h0->size2);
  gsl_multifit_linear(par_h0, obs, c, cov, &res0, ws);
  gsl_vector_free(c);
  gsl_matrix_free(cov);
  gsl_multifit_linear_free(ws);
  ws=gsl_multifit_linear_alloc(obs->size, par_h1->size2);
  c=gsl_vector_alloc(par_h1->size2);
  cov=gsl_matrix_alloc(par_h1->size2, par_h1->size2);
  gsl_multifit_linear(par_h1, obs, c, cov, &res1, ws);
  gsl_vector_free(c);
  gsl_matrix_free(cov);
  gsl_multifit_linear_free(ws);
  fd0=obs->size-par_h0->size2;
  fd1=obs->size-par_h1->size2;
  double f;
  f=((res0-res1)/(fd0-fd1))/(res1/fd1);
  double pval;
  pval = gsl_cdf_fdist_Q(f, fd0-fd1, fd1);
  return pval;
}
