#include "basic_type.hpp"

bool ID_index::add(string &id, size_t index)
{
  if (id=="")
  {
    printf("Error: NULL probe id\n");
    return false;
  }
  if (index_to_id.find(index)!=index_to_id.end())
  {
    printf("Error: duplicated probe index %d\n",index);
    return false;
  }
  if (id_to_index.find(id)!=id_to_index.end())
  {
    printf("Warning: duplicated probe id %s, overwrite old one\n", id.c_str());
  }
  index_to_id.insert(map<size_t, string>::value_type(make_pair(index, id)));
  id_to_index.insert(map<string, size_t>::value_type(make_pair(id, index)));
  return true;
}

size_t ID_index::get(string &id)
{
  map<string, size_t>::iterator pos;
  pos=id_to_index.find(id);
  if (pos==id_to_index.end())
  {
    printf("Error: no such probe %s\n", id.c_str());
    return 0;
  }
  return pos->second;
}

string ID_index::get(size_t index)
{
  map<size_t, string>::iterator pos;
  string tmp;
  pos=index_to_id.find(index);
  if (pos==index_to_id.end())
  {
    printf("Error: wrong probe index %d\n", index);
    return tmp;
  }
  return pos->second;
}

size_t ID_index::size()
{
  return id_to_index.size();
}

bool Sample_group::add(string &sample, string &group)
{
  vector<string> tmp;
  if (sample=="")
  {
    printf("Error: NULL sample name\n");
    return false;
  }
  if (group=="")
  {
    printf("Error: NULL group name\n");
  }
  map<string, string>::iterator pos;
  pos=sample_group.find(sample);
  if (pos!=sample_group.end())
  {
    printf("Error: duplicate sample name %s\n", sample.c_str());
    return false;
  }
  sample_group.insert(map<string, string>::value_type(make_pair(sample,group)));
  if (group_sample.find(group)!=group_sample.end())
  {
    group_sample[group].push_back(sample);
  }
  else
  {
    group_sample.insert(map<string, vector<string> >::value_type(group,tmp));
    group_sample[group].push_back(sample);
  }
  return true;
}

string Sample_group::get_group(string &sample)
{
  string tmp;
  map<string, string>::iterator pos;
  pos=sample_group.find(sample);
  if (pos==sample_group.end())
  {
    printf("Error: no group found of  sample %s\n",sample.c_str());
    return tmp;
  }
  return pos->second;
}

vector<string> Sample_group::get_sample(string &group)
{
  vector<string> tmp;
  map<string, vector<string> >::iterator pos;
  pos=group_sample.find(group);
  if (pos==group_sample.end())
  {
    printf("Error: no sample found of group %s\n", group.c_str());
    return tmp;
  }
  return pos->second;
}

size_t Sample_group::group_num()
{
  return group_sample.size();
}

void Order::set(string &str, double dbl)
{
  this->str=str;
  this->dbl=dbl;
}

bool Order::operator<(const Order &x) const
{
  if (this->dbl<x.dbl)
  {
    return true;
  }
  if (this->dbl>x.dbl)
  {
    return false;
  }
  return this->str < x.str;
}
