#include "dmr_analysis.hpp"

#define LENGTH 100
#define THRESHOLD 0.05

inline size_t probe_distance(vector<Point> &ann, size_t i, size_t j)
{
  if (ann[i].chromosome!=ann[j].chromosome)
  {
    return LENGTH+1;
  }
  else
  {
    return ann[j].position-ann[i].position;
  }
}

void find_region(vector<Point> &ann, map<string, size_t> &map_probe, CGI_description &cgi_des)
{
  size_t i;
  size_t j;
  Bed tmp_bed;
  vector<string> tmp_vec;
  for(i=0; i<ann.size()-1; i++)
  {
    tmp_vec.clear();
    j=i+1;
    tmp_bed.start=ann[i].position;
    while(j<ann.size() && probe_distance(ann, i, j)<=LENGTH)
    {
      tmp_vec.push_back(ann[i].name);
      i=j;
      j++;
    }
    if (!tmp_vec.empty())
    {
      tmp_vec.push_back(ann[i].name);
      tmp_bed.end=ann[j-1].position+1;
      tmp_bed.chromosome=ann[i].chromosome;
      cgi_des.cgi.push_back(tmp_bed);
      cgi_des.cgi_probe.insert(map<size_t, vector<string> >::value_type(make_pair(cgi_des.cgi.size()-1, tmp_vec)));
    }
  }
}

void dmr_analysis(Data_description &data_des,
    Data_matrix *data,
    Parameter &param,
    vector<Point> &ann,
    map<string, size_t> &map_probe,
    CGI_description &cgi_des,
    CGI_result &cgi_res)
{
  size_t i;
  size_t j;
  double sum;
  size_t s;
  size_t e;
  Bed tmp_bed;
  string tmp_str;
  size_t tmp_index;
  vector<string> tmp_vec;
  vector<size_t> dmr_vec;
  set<string> tmp_set;
  set<string>::iterator its;
  time_out();
  printf("     Analyzing differentially methylated arbitrary region\n");
  find_region(ann, map_probe, cgi_des);
  cgi_res.pvalue=gsl_vector_alloc(cgi_des.cgi.size());
  cgi_res.fdr=gsl_vector_alloc(cgi_des.cgi.size());
  gsl_vector_set_all(cgi_res.pvalue, 2.0);
  gsl_vector_set_all(cgi_res.fdr, 2.0);
  boost::thread_group *thrdg;
  boost::thread *thrd;
  size_t remainder;
  thrdg=new boost::thread_group;
  for (remainder=0; remainder<param.nproc; remainder++)
  {
    thrd=new boost::thread(cgi_analysis_thread, data, remainder, param.nproc, &data_des, &cgi_des, &cgi_res);
    thrdg->add_thread(thrd);
  }
  thrdg->join_all();
  time_out();
  printf("       Finding fdr in arbitrary region analysis\n");
  fdr_range_cgi(cgi_res, cgi_des, data_des);
}
