#include "parameter.hpp"

#define VERSION "1.2.1"
#define SOFTWARE "FastDMA"


Parameter::Parameter()
{
  pvalue_threshold=0.01;
  fdr_threshold=0.01;
  covariable_number=0;
  nproc=1;
  normalize=true;
  cgi=true;
  dmr=true;
  continuous=false;
  snp=false;
  probe_test_file="probe.txt";
  cgi_test_file="r1.txt";
  dmr_test_file="r2.txt";
  configure_file="../etc/description.xls";
  cgi_bedgraph_prefix="r1";
  dmr_bedgraph_prefix="r2";
  probe_bedgraph_prefix="probe";
  probe_ann_file="../data/probe.ann";
  cgi_file="../data/cpgIslandExt.txt";
}

void help()
{
  cout << SOFTWARE << " " << VERSION <<endl << endl;
  cout << "Usage:" <<endl;
  cout << "-h --help                                                print this help info" << endl<<endl;
  cout << "<optional steps choice>"<<endl;
  cout << "--continuous                                             analyze relationship with continous variable rather than group"<<endl;
  cout << "--nosnp                                                  filter out probes containing genomic variance"<<endl;
  cout << "--normalize-off                                          do not normalize the raw betas" << endl;
  cout << "--r1-off                                                 do not analyze predefined region" << endl;
  cout << "--r2-off                                                 do not analyze arbitrary region" << endl<<endl;
  cout << "<threshold choice>"<<endl;
  cout << "-n --thread                   [int]                      set threads number" << endl;
  cout << "-p --pvalue                   [double]                   set p value threshold" << endl;
  cout << "-q --fdr                      [double]                   set fdr threshold" << endl<<endl;
  cout << "<basic function configuration>"<<endl;
  cout << "-c --configure                [string]                   configure file" << endl;
  cout << "--covariable                  [int]                      covariable number" << endl;
  cout << "--probe                       [string]                   probe annotation file"<<endl;
  cout << "--probe-prefix                [string]                   probe BedGraph result file prefix" << endl<<endl;
  cout << "<predefined region configuration>"<<endl;
  cout << "--r1                          [string]                   predefined region file"<<endl;
  cout << "--r1-prefix                   [string]                   predefined region Bedgraph result file prefix" << endl<<endl;
  cout << "<arbitrary region configuration>"<<endl;
  cout << "--r2-prefix                   [string]                   arbitrary region Bedgraph result file prefix" << endl;
}

Parameter parameter_manipulate(int argc, char *argv[])
{
  Parameter param;
  size_t i;
  string val;
  string op;
  stringstream sstrm;
  if (argc==1)
  {
    help();
    exit(-1);
  }
  for (i=1; i<argc; i++)
  {
    op=argv[i];
    if (op=="-h" || op=="--help")
    {
      help();
      exit(0);
    }
    else if (op=="--normalize-off")
    {
      param.normalize=false;
    }
    else if (op=="--nosnp")
    {
      param.snp=true;
    }
    else if (op=="--r1-off")
    {
      param.cgi=false;
    }
    else if (op=="--r2-off")
    {
      param.dmr=false;
    }
    else if (op=="--continuous")
    {
      param.continuous=true;
    }
    else if (op=="-n" || op=="--thread")
    {
      sstrm.clear();
      i++;
      if (i==argc)
      {
        printf("Lack value of %s\n", argv[i-1]);
        help();
        exit(-1);
      }
      sstrm<<argv[i];
      sstrm>>param.nproc;
      if (!sstrm || param.nproc<1)
      {
        printf("Invalid nproc value %s\n", argv[i]);
        help();
        exit(-1);
      }
    }
    else if (op=="-p" || op=="--pvalue")
    {
      sstrm.clear();
      i++;
      if (i==argc)
      {
        printf("Lack value of %s\n", argv[i-1]);
        help();
        exit(-1);
      }
      sstrm<<argv[i];
      sstrm>>param.pvalue_threshold;
      if (!sstrm || param.pvalue_threshold<0.0 || param.pvalue_threshold>1.01)
      {
        printf("Invalid pvalue %s\n", argv[i]);
        help();
        exit(-1);
      }
    }
    else if (op=="-q" || op=="--fdr")
    {
      sstrm.clear();
      i++;
      if (i==argc)
      {
        printf("Lack value of %s\n", argv[i-1]);
        help();
        exit(-1);
      }
      sstrm<<argv[i];
      sstrm>>param.fdr_threshold;
      if (!sstrm || param.fdr_threshold<0.0 || param.fdr_threshold>1.01)
      {
        printf("Invalid fdr value %s\n", argv[i]);
        help();
        exit(-1);
      }
    }
    else if (op=="--covariable")
    {
      i++;
      if (i==argc)
      {
        printf("Lack value of %s\n", argv[i-1]);
        help();
        exit(-1);
      }
      sstrm.clear();
      sstrm.str("");
      sstrm<<argv[i];
      sstrm>>param.covariable_number;
      if (!sstrm)
      {
        printf("Invalid range value %s\n", argv[i]);
        help();
        exit(-1);
      }
    }
    else if (op == "--probe-prefix")
    {
      i++;
      if (i==argc)
      {
        cout<<"Lack value of "<<argv[i-1]<<endl;
        help();
        exit(-1);
      }
      sstrm.clear();
      sstrm.str("");
      sstrm<<argv[i];
      sstrm>>param.probe_bedgraph_prefix;
      if (!sstrm)
      {
        cout<<"Invalid file name"<<argv[i]<<endl;
        help();
        exit(-1);
      }
    }
    else if (op == "-c" || op=="--configure")
    {
      i++;
      if (i==argc)
      {
        printf("Lack value of %s\n", argv[i-1]);
        help();
        exit(-1);
      }
      param.configure_file=argv[i];
    }
    else if (op=="--r1")
    {
      i++;
      if (i==argc)
      {
        printf("Lack value of --r1\n");
        help();
        exit(-1);
      }
      param.cgi_file=argv[i];
    }
    else if (op=="--probe")
    {
      i++;
      if (i==argc)
      {
        printf("Lack value of --probe\n");
        help();
        exit(-1);
      }
      param.probe_ann_file=argv[i];
    }
    else if (op=="--r1-prefix")
    {
      i++;
      if (i==argc)
      {
        printf("Lack value of --r1-prefix");
        help();
        exit(-1);
      }
      param.cgi_bedgraph_prefix=argv[i];
    }
    else if (op=="--r2-prefix")
    {
      i++;
      if (i==argc)
      {
        printf("Lack value of --r2-prefix\n");
        help();
        exit(-1);
      }
      param.dmr_bedgraph_prefix=argv[i];
    }
    else
    {
      printf("Unrecognized parameter %s\n", argv[i]);
      help();
      exit(-1);
    }
  }
  param.probe_test_file=param.probe_bedgraph_prefix+".txt";
  param.cgi_test_file=param.cgi_bedgraph_prefix+".txt";
  param.dmr_test_file=param.dmr_bedgraph_prefix+".txt";
  return param;
}
