#include "read_file.hpp"

#define LINE 65536

bool initialize_configure(string &configure_file_name,
                          Data_description &data_dsp,
                          Data_matrix *data,
                          size_t &probe_number,
                          size_t &sample_number,
                          size_t &covariable_number,
                          Parameter &param)
{
  FILE *fp_configure;
  FILE *fp_data;
  if (!(fp_configure=fopen(configure_file_name.c_str(),"r")))
  {
    printf("Error: cannot open file %s.\n", configure_file_name.c_str());
    return false;
  }
  stringstream strm;
  char buff[LINE];
  string data_file_name;
  string sample_name;
  string group_name;
  sample_number=0;
  double continuous_variable;
  size_t group_num;
  group_num=0;
  double var;
  size_t i, j;
  map<string, size_t> gi;
  while(true)
  {
    if (!fgets(buff, LINE-1, fp_configure))
    {
      break;
    }
    if (buff[0]=='#')
      continue;
    strm.clear();
    strm.str("");
    strm<<buff;
    if (!param.continuous)
    {
      strm>>data_file_name>>sample_name>>group_name;
    }
    else
    {
      strm>>data_file_name>>sample_name>>continuous_variable;
      group_name.assign("all");
    }
    if (!strm)
    {
      cout<<"Error: wrong format when reading configuration file"<<configure_file_name.c_str()<<endl<<"Wrong sentence: "<<buff<<endl;
    }
    if (gi.find(group_name)==gi.end())
    {
      gi.insert(map<string, size_t>::value_type(make_pair(group_name, group_num)));
      data_dsp.group_index.add(group_name,group_num);
      group_num++;
    }
    data_dsp.sample_group.add(sample_name, group_name);
    data_dsp.sample_index.add(sample_name, sample_number);
    sample_number++;
  }
  fclose(fp_configure);
  if (! (fp_data=fopen(data_file_name.c_str(), "r")))
  {
    printf("Error: cannot open file %s\n", data_file_name.c_str());
    return false;
  }
  probe_number=0;
  while(true)
  {
    if (! fgets(buff, LINE-1, fp_data))
    {
      break;
    }
    strm.clear();
    strm.str("");
    strm<<buff;
    strm>>sample_name;
    data_dsp.probe_index.add(sample_name, probe_number);
    probe_number++;
  }
  fstream fs;
  if (param.continuous)
  {
    data->continuous=gsl_matrix_alloc(sample_number, 1);
    fs.open(configure_file_name.c_str(), ios_base::in);
    i=0;
    while(true)
    {
      fs.getline(buff, LINE-1);
      if (!fs)
        break;
      if (buff[0]=='#')
        continue;
      strm.clear();
      strm.str("");
      strm<<buff;
      strm>>data_file_name>>sample_name>>continuous_variable;
      gsl_matrix_set(data->continuous, i, 0, continuous_variable);
      i++;
    }
    fs.close();
  }
  else
  {
    data->continuous=0;
  }
  if (covariable_number == 0)
  {
    data->covariable=0;
  }
  else
  {
    data->covariable=gsl_matrix_alloc(sample_number, covariable_number);
    i=0;
    fs.open(configure_file_name.c_str(), ios_base::in);
    while(true)
    {
      fs.getline(buff, LINE-1);
      if (!fs)
        break;
      if (buff[0]=='#')
        continue;
      strm.clear();
      strm.str("");
      strm<<buff;
      strm>>data_file_name>>sample_name>>group_name;
      for (j=0; j<covariable_number; j++)
      {
        strm>>var;
        gsl_matrix_set(data->covariable, i, j, var);
      }
      i++;
    }
    fs.close();
  }
  return true;
}

bool read_data_thread(vector<string> *data_file,
                      Data_matrix *data,
                      size_t remainder,
                      size_t nproc)
{
  FILE *fp;
  size_t i;
  size_t j;
  stringstream sstrm;
  char buff[LINE];
  char id[LINE];
  char beta_str[LINE];
  char pval_str[LINE];
  string probe_id;
  double beta;
  double pval;
  for (i=remainder; i<(*data_file).size(); i+=nproc)
  {
    if (! (fp=fopen((*data_file)[i].c_str(), "r")))
    {
      printf("Error: cannot open file %s\n", (*data_file)[i].c_str());
      return false;
    }
    j=0;
    while(true)
    {
      if (!fgets(buff, LINE-1, fp))
      {
        break;
      }
      sstrm.clear();
      sstrm.str("");
      sstrm<<buff;
      sstrm>>probe_id;
      sstrm>>beta;
      sstrm>>pval;
      if (!sstrm)
      {
        cout << "Error: wrong format when reading data file"<<endl<<"Error sentence: "<<buff<<endl;
        beta=-1.0;
        gsl_matrix_set(data->valid, j, i, -1.0);
      }
      if (pval>0.05)
      {
        beta=0.5;
        gsl_matrix_set(data->valid, j, i, -1.0);
      }
      if (beta<0.0 || beta>1.0)
      {
        cout<<"Error: wrong beta value encountered when reading data"<<endl<<"Wrong sentence: "<<buff<<endl;
      }
      gsl_matrix_set(data->data, j, i, log(beta/(1-beta)));
      gsl_matrix_set(data->valid, j, i, 1.0);
      j++;
    }
    fclose(fp);
  }
}

Data_matrix* read_data_file(string &configure_file_name,
                    Data_matrix *data,
                    size_t &probe_number,
                    size_t &sample_number,
                    size_t nproc)
{
  data->data=gsl_matrix_alloc(probe_number, sample_number);
  data->valid=gsl_matrix_alloc(probe_number, sample_number);
  //data=gsl_matrix_alloc(probe_number, sample_number);
  boost::thread_group *thrdg;
  boost::thread *thrd;
  size_t i;
  vector<string> data_file;
  char buff[LINE];
  char fname[LINE];
  string tmp;
  FILE *fp;
  if (! (fp=fopen(configure_file_name.c_str(), "r")))
  {
    printf("Error: cannot open file %s\n", configure_file_name.c_str());
    gsl_matrix_free(data->data);
    gsl_matrix_free(data->valid);
    if (data->covariable!=0)
    {
      gsl_matrix_free(data->covariable);
    }
    return 0;
  }
  while(true)
  {
    if (! fgets(buff, LINE-1, fp))
    {
      break;
    }
    if (buff[0]=='#')
      continue;
    sscanf(buff, "%s\t", fname);
    tmp=fname;
    data_file.push_back(fname);
  }
  thrdg=new boost::thread_group;
  for (i=0; i<nproc; i++)
  {
    thrd=new boost::thread(read_data_thread,
                           &data_file,
                           data,
                           i,
                           nproc);
    thrdg->add_thread(thrd);
  }
  thrdg->join_all();
  fclose(fp);
  delete thrdg;
  return data;
}

Data_matrix* read_data(string &configure_file_name,
                       Data_description &data_dsp,
                       size_t covariable_number,
                       size_t nproc,
                       Parameter &param)
{
  size_t probe_number;
  size_t sample_number;
  time_out();printf("     ");
  printf("Read data\n");
  Data_matrix *data=new(Data_matrix);
  initialize_configure(configure_file_name,
                       data_dsp,
                       data,
                       probe_number,
                       sample_number,
                       covariable_number,
                       param);
  read_data_file(configure_file_name,
                 data,
                        probe_number,
                        sample_number,
                        nproc);
  size_t i,j,k;
  string id;
  if (param.snp)
  {
    for(i=0; i<data_dsp.probe_index.size(); i++)
    {
      id=data_dsp.probe_index.get(i);
      if (id.substr(0,2)=="rs")
      {
        for(j=0; j<data->valid->size2; j++)
        {
          gsl_matrix_set(data->valid,i,j,-1.0);
        }
      }
    }
  }
  return data;
}
